@@ -24,22 +24,20 @@
 
 #include "DebconfElement.h"
 
-namespace DebconfKde {
+namespace DebconfKde
+{
 
 class DebconfBoolean : public DebconfElement, Ui::DebconfBoolean
 {
     Q_OBJECT
 public:
-    explicit DebconfBoolean(const QString &name, QWidget *parent = 0);
+    explicit DebconfBoolean(const QString &name, QWidget *parent = nullptr);
     ~DebconfBoolean();
 
-    void setBoolean(const QString &extended_description,
-                    const QString &description,
-                    bool default_boolean);
+    void setBoolean(const QString &extended_description, const QString &description, bool default_boolean);
     QString value() const override;
 };
 
-
 }
 
 #endif
