@@ -24,20 +24,20 @@
 
 #include "DebconfElement.h"
 
-namespace DebconfKde {
+namespace DebconfKde
+{
 
 class DebconfError : public DebconfElement, Ui::DebconfError
 {
     Q_OBJECT
 public:
-    explicit DebconfError(const QString &name, QWidget *parent = 0);
+    explicit DebconfError(const QString &name, QWidget *parent = nullptr);
     ~DebconfError();
 
     void setError(const QString &description, const QString &title);
     int nextId() const;
 };
 
-
 }
 
 #endif
