@@ -63,14 +63,17 @@
 #include "DebconfText.h"
 #include "Debug_p.h"
 
+#include <QFrame>
+#include <QHostInfo>
 #include <QtCore/QDebug>
-#include <QtCore/QProcess>
 #include <QtCore/QFile>
+#include <QtCore/QProcess>
 #include <QtWidgets/QLabel>
 
 #include <KGuiItem>
 #include <KIconLoader>
 #include <KLocalizedString>
+#include <KOSRelease>
 #include <KStandardGuiItem>
 
 #include <debconf.h>
@@ -80,19 +83,21 @@ using namespace DebconfKde;
 class DebconfKde::DebconfGuiPrivate : public Ui::DebconfGui
 {
 public:
-    virtual ~DebconfGuiPrivate() { }
+    virtual ~DebconfGuiPrivate()
+    {
+    }
     DebconfFrontend *frontend;
     DebconfProgress *elementProgress = nullptr;
     QWidget *parentWidget = nullptr;
-    QVector<DebconfElement*> elements;
+    QVector<DebconfElement *> elements;
 
-    DebconfElement* createElement(const QString &k);
+    DebconfElement *createElement(const QString &k);
     void cleanup();
 };
 
 DebconfGui::DebconfGui(const QString &socketName, QWidget *parent)
- : QWidget(parent),
-   d_ptr(new DebconfGuiPrivate)
+    : QWidget(parent)
+    , d_ptr(new DebconfGuiPrivate)
 {
     Q_D(DebconfGui);
     d->frontend = new DebconfFrontendSocket(socketName, this);
@@ -100,8 +105,8 @@ DebconfGui::DebconfGui(const QString &socketName, QWidget *parent)
 }
 
 DebconfGui::DebconfGui(int readfd, int writefd, QWidget *parent)
- : QWidget(parent),
-   d_ptr(new DebconfGuiPrivate)
+    : QWidget(parent)
+    , d_ptr(new DebconfGuiPrivate)
 {
     Q_D(DebconfGui);
     d->frontend = new DebconfFrontendFifo(readfd, writefd, this);
@@ -131,30 +136,15 @@ void DebconfGui::init()
     connect(d->frontend, &DebconfFrontend::progress, this, &DebconfGui::cmd_progress);
     connect(d->frontend, &DebconfFrontend::backup, d->backPB, &QPushButton::setEnabled);
 
+    setWindowTitle(i18n("Debconf on %1", QHostInfo::localHostName()));
+
     // find out the distribution logo
-    QString distro_logo(QLatin1String("/usr/share/pixmaps/debian-logo.png"));
-    auto myProcess = new QProcess(this);
+    QString distroLogo(QLatin1String("/usr/share/pixmaps/debian-logo.png"));
+    KOSRelease osInfo;
+    if (!osInfo.logo().isEmpty())
+        distroLogo = osInfo.logo();
 
-    myProcess->start(QLatin1String("hostname"));
-    myProcess->waitForFinished();
-    const QString hostname = QString::fromLatin1(myProcess->readAllStandardOutput());
-    setWindowTitle(i18n("Debconf on %1", hostname.trimmed()));
-
-    myProcess->start(QLatin1String("lsb_release"), { QLatin1String("-is") });
-    if (myProcess->waitForFinished()) {
-        if (myProcess->exitCode() == 0){
-            QString data = QString::fromLatin1(myProcess->readAllStandardOutput());
-            data = QString(QLatin1String("/usr/share/pixmaps/%1-logo.png")).arg(data.trimmed().toLower());
-            if (QFile::exists(data)) {
-                distro_logo = data;
-            }
-        }
-    }
-
-    const QPixmap icon = KIconLoader::global()->loadIcon(distro_logo,
-                                                         KIconLoader::NoGroup,
-                                                         KIconLoader::SizeLarge,
-                                                         KIconLoader::DefaultState);
+    const QPixmap icon = KIconLoader::global()->loadIcon(distroLogo, KIconLoader::NoGroup, KIconLoader::SizeLarge, KIconLoader::DefaultState);
     if (!icon.isNull()) {
         d->iconL->setPixmap(icon);
         setWindowIcon(icon);
@@ -163,7 +153,7 @@ void DebconfGui::init()
     d->scrollArea->viewport()->setAutoFillBackground(false);
 }
 
-DebconfElement* DebconfGuiPrivate::createElement(const QString &k)
+DebconfElement *DebconfGuiPrivate::createElement(const QString &k)
 {
     qCDebug(DEBCONF) << "creating widget for " << k;
 
@@ -171,23 +161,17 @@ DebconfElement* DebconfGuiPrivate::createElement(const QString &k)
     extendedDescription.replace(QLatin1String("\\n"), QLatin1String("\n"));
 
     switch (frontend->type(k)) {
-    case DebconfFrontend::Boolean:
-    {
+    case DebconfFrontend::Boolean: {
         auto element = new DebconfBoolean(k, parentWidget);
-        element->setBoolean(extendedDescription,
-                            frontend->property(k, DebconfFrontend::Description),
-                            frontend->value(k) == QLatin1String("true"));
+        element->setBoolean(extendedDescription, frontend->property(k, DebconfFrontend::Description), frontend->value(k) == QLatin1String("true"));
         return element;
     }
-    case DebconfFrontend::Error:
-    {
+    case DebconfFrontend::Error: {
         auto element = new DebconfError(k, parentWidget);
-        element->setError(extendedDescription,
-                          frontend->property(k, DebconfFrontend::Description));
+        element->setError(extendedDescription, frontend->property(k, DebconfFrontend::Description));
         return element;
     }
-    case DebconfFrontend::Multiselect:
-    {
+    case DebconfFrontend::Multiselect: {
         auto element = new DebconfMultiselect(k, parentWidget);
         element->setMultiselect(extendedDescription,
                                 frontend->property(k, DebconfFrontend::Description),
@@ -195,22 +179,17 @@ DebconfElement* DebconfGuiPrivate::createElement(const QString &k)
                                 frontend->property(k, DebconfFrontend::Choices).split(QLatin1String(", ")));
         return element;
     }
-    case DebconfFrontend::Note:
-    {
+    case DebconfFrontend::Note: {
         auto element = new DebconfNote(k, parentWidget);
-        element->setNote(extendedDescription,
-                         frontend->property(k, DebconfFrontend::Description));
+        element->setNote(extendedDescription, frontend->property(k, DebconfFrontend::Description));
         return element;
     }
-    case DebconfFrontend::Password:
-    {
+    case DebconfFrontend::Password: {
         auto element = new DebconfPassword(k, parentWidget);
-        element->setPassword(extendedDescription,
-                             frontend->property(k, DebconfFrontend::Description));
+        element->setPassword(extendedDescription, frontend->property(k, DebconfFrontend::Description));
         return element;
     }
-    case DebconfFrontend::Select:
-    {
+    case DebconfFrontend::Select: {
         auto element = new DebconfSelect(k, parentWidget);
         element->setSelect(extendedDescription,
                            frontend->property(k, DebconfFrontend::Description),
@@ -218,16 +197,12 @@ DebconfElement* DebconfGuiPrivate::createElement(const QString &k)
                            frontend->property(k, DebconfFrontend::Choices).split(QLatin1String(", ")));
         return element;
     }
-    case DebconfFrontend::String:
-    {
+    case DebconfFrontend::String: {
         auto element = new DebconfString(k, parentWidget);
-        element->setString(extendedDescription,
-                           frontend->property(k, DebconfFrontend::Description),
-                           frontend->value(k));
+        element->setString(extendedDescription, frontend->property(k, DebconfFrontend::Description), frontend->value(k));
         return element;
     }
-    case DebconfFrontend::Text:
-    {
+    case DebconfFrontend::Text: {
         auto element = new DebconfText(k, parentWidget);
         element->setText(frontend->property(k, DebconfFrontend::Description));
         return element;
@@ -236,10 +211,11 @@ DebconfElement* DebconfGuiPrivate::createElement(const QString &k)
         qWarning() << "Default REACHED!!!";
         auto element = new DebconfElement(k, parentWidget);
         auto label = new QLabel(element);
-        label->setText(i18n("<b>Not implemented</b>: The input widget for data"
-                            " type '%1' is not implemented. Will use default of '%2'.",
-                            frontend->property(k, DebconfFrontend::Type),
-                            frontend->value(k)));
+        label->setText(
+            i18n("<b>Not implemented</b>: The input widget for data"
+                 " type '%1' is not implemented. Will use default of '%2'.",
+                 frontend->property(k, DebconfFrontend::Type),
+                 frontend->value(k)));
         label->setWordWrap(true);
         return element;
     }
@@ -250,18 +226,22 @@ void DebconfGui::cmd_go(const QString &title, const QStringList &input)
     Q_D(DebconfGui);
     qCDebug(DEBCONF) << "# GO GUI";
     d->cleanup();
-    auto layout = qobject_cast<QVBoxLayout*>(d->parentWidget->layout());
+    auto layout = qobject_cast<QVBoxLayout *>(d->parentWidget->layout());
     // if we have just one element and we are showing
     // elements that can make use of extra space
     // we don't add stretches
     bool needStretch = true;
     if (input.size() == 1) {
         const QString key = input.first();
-        if (d->frontend->type(key) == DebconfFrontend::Text &&
-            d->frontend->type(key) == DebconfFrontend::Note &&
-            d->frontend->type(key) == DebconfFrontend::Error &&
-            d->frontend->type(key) == DebconfFrontend::Multiselect) {
+        switch (d->frontend->type(key)) {
+        case DebconfFrontend::Text:
+        case DebconfFrontend::Note:
+        case DebconfFrontend::Error:
+        case DebconfFrontend::Multiselect:
             needStretch = false;
+            break;
+        default:
+            break;
         }
     }
 
@@ -269,8 +249,21 @@ void DebconfGui::cmd_go(const QString &title, const QStringList &input)
         layout->addStretch();
     }
 
+    const QString side_info = d->frontend->getSideInfo();
+    if (!side_info.isEmpty()) {
+        QLabel *label = new QLabel(side_info);
+        layout->addWidget(label);
+    }
+
     for (const QString &elementName : input) {
-        DebconfElement *element =  d->createElement(elementName);
+        if (elementName == QLatin1String("div")) {
+            QFrame *divider = new QFrame(d->parentWidget);
+            divider->setFrameShape(QFrame::HLine);
+            divider->setFrameShadow(QFrame::Sunken);
+            layout->addWidget(divider);
+            continue;
+        }
+        DebconfElement *element = d->createElement(elementName);
         d->elements.append(element);
         layout->addWidget(element);
     }
@@ -292,7 +285,7 @@ void DebconfGui::cmd_progress(const QString &cmd)
     if (!d->elementProgress) {
         d->cleanup();
         d->elementProgress = new DebconfProgress(QString(), d->parentWidget);
-        auto layout = qobject_cast<QVBoxLayout*>(d->parentWidget->layout());
+        auto layout = qobject_cast<QVBoxLayout *>(d->parentWidget->layout());
         layout->addStretch();
         layout->addWidget(d->elementProgress);
         layout->addStretch();
@@ -308,9 +301,7 @@ void DebconfGui::cmd_progress(const QString &cmd)
         d->titleL->setText(d->frontend->property(commands.at(3), DebconfFrontend::Description));
         int progress_min = commands.at(1).toInt();
         int progress_max = commands.at(2).toInt();
-        element->startProgress(d->frontend->property(commands.at(3), DebconfFrontend::ExtendedDescription),
-                               progress_min,
-                               progress_max);
+        element->startProgress(d->frontend->property(commands.at(3), DebconfFrontend::ExtendedDescription), progress_min, progress_max);
     } else if (commands.first() == QLatin1String("SET")) {
         element->setProgress(commands.at(1).toInt());
     } else if (commands.first() == QLatin1String("STEP")) {
