@@ -53,7 +53,10 @@
 
 #include <QWidget>
 
-namespace DebconfKde {
+#include "debconf-kde_export.h"
+
+namespace DebconfKde
+{
 
 /**
  * \class DebconfGui DebconfGui.h DebconfGui
@@ -83,7 +86,7 @@ namespace DebconfKde {
  * Only delete it after you are sure no more operations ended.
  */
 class DebconfGuiPrivate;
-class Q_DECL_EXPORT DebconfGui : public QWidget
+class DEBCONF_KDE_EXPORT DebconfGui : public QWidget
 {
     Q_OBJECT
 public:
@@ -93,14 +96,14 @@ public:
      * \warning Be adivised that this class will delete the path pointed
      * by \p socketName. A good location would be /tmp/debconf-$PID.
      */
-    explicit DebconfGui(const QString &socketName, QWidget *parent = 0);
+    explicit DebconfGui(const QString &socketName, QWidget *parent = nullptr);
 
     /**
      * Constructor that prepares for communication with Debconf via FIFO pipes.
      * Read (\p readfd) and write (\p writefd) file descriptors should be open
      * and connected to Debconf which speaks Passthrough frontend protocol.
      */
-    explicit DebconfGui(int readfd, int writefd, QWidget *parent = 0);
+    explicit DebconfGui(int readfd, int writefd, QWidget *parent = nullptr);
 
     ~DebconfGui();
 
@@ -134,19 +137,18 @@ protected:
      */
     void closeEvent(QCloseEvent *event) override;
 
-    DebconfGuiPrivate * const d_ptr;
+    DebconfGuiPrivate *const d_ptr;
 
 private:
     Q_DECLARE_PRIVATE(DebconfGui)
 
     /**
-      * This routine is called by all constructors to perform common
-      * initialization.
-      */
+     * This routine is called by all constructors to perform common
+     * initialization.
+     */
     void init();
 };
 
-
 }
 
 #endif
