@@ -26,26 +26,23 @@
 
 class QStandardItemModel;
 
-namespace DebconfKde {
+namespace DebconfKde
+{
 
 class DebconfMultiselect : public DebconfElement, Ui::DebconfMultiselect
 {
     Q_OBJECT
 public:
-    explicit DebconfMultiselect(const QString &name, QWidget *parent = 0);
+    explicit DebconfMultiselect(const QString &name, QWidget *parent = nullptr);
     ~DebconfMultiselect();
 
-    void setMultiselect(const QString &extended_description,
-                        const QString &description,
-                        const QStringList &default_choices,
-                        const QStringList &choices);
+    void setMultiselect(const QString &extended_description, const QString &description, const QStringList &default_choices, const QStringList &choices);
     QString value() const override;
 
 private:
     QStandardItemModel *m_model;
 };
 
-
 }
 
 #endif
