@@ -24,20 +24,20 @@
 
 #include "DebconfElement.h"
 
-namespace DebconfKde {
+namespace DebconfKde
+{
 
 class DebconfPassword : public DebconfElement, Ui::DebconfPassword
 {
     Q_OBJECT
 public:
-    explicit DebconfPassword(const QString &name, QWidget *parent = 0);
+    explicit DebconfPassword(const QString &name, QWidget *parent = nullptr);
     ~DebconfPassword();
 
     void setPassword(const QString &extended_description, const QString &description);
     QString value() const override;
 };
 
-
 }
 
 #endif
