@@ -24,25 +24,23 @@
 
 #include "DebconfElement.h"
 
-namespace DebconfKde {
+namespace DebconfKde
+{
 
 class DebconfProgress : public DebconfElement, Ui::DebconfProgress
 {
     Q_OBJECT
 public:
-    explicit DebconfProgress(const QString &name, QWidget *parent = 0);
+    explicit DebconfProgress(const QString &name, QWidget *parent = nullptr);
     ~DebconfProgress();
 
-    void startProgress(const QString &extended_description,
-                       uint progress_min,
-                       uint progress_max);
+    void startProgress(const QString &extended_description, uint progress_min, uint progress_max);
     void stopProgress();
     void setProgress(uint progress_cur);
     void stepProgress(uint progress_cur);
     void setProgressInfo(const QString &description);
 };
 
-
 }
 
 #endif
