@@ -24,18 +24,17 @@
 
 #include "DebconfElement.h"
 
-namespace DebconfKde {
+namespace DebconfKde
+{
 
 class DebconfString : public DebconfElement, Ui::DebconfString
 {
     Q_OBJECT
 public:
-    explicit DebconfString(const QString &name, QWidget *parent = 0);
+    explicit DebconfString(const QString &name, QWidget *parent = nullptr);
     ~DebconfString();
 
-    void setString(const QString &extended_description,
-                   const QString &description,
-                   const QString &default_string);
+    void setString(const QString &extended_description, const QString &description, const QString &default_string);
     QString value() const override;
 };
 
