@@ -51,40 +51,43 @@
 
 #include "debconf.h"
 
-#include <QtCore/QSocketNotifier>
-#include <QtCore/QRegExp>
 #include <QtCore/QFile>
+#include <QtCore/QRegularExpression>
+#include <QtCore/QSocketNotifier>
 #include <cstdio>
 #include <unistd.h>
 
 #include "Debug_p.h"
 
-namespace DebconfKde {
+namespace DebconfKde
+{
 
-const DebconfFrontend::Cmd DebconfFrontend::commands[] = {
-    { "SET", &DebconfFrontend::cmd_set },
-    { "GO", &DebconfFrontend::cmd_go },
-    { "TITLE", &DebconfFrontend::cmd_title },
-    { "SETTITLE", &DebconfFrontend::cmd_title },
-    { "DATA", &DebconfFrontend::cmd_data },
-    { "SUBST", &DebconfFrontend::cmd_subst },
-    { "INPUT", &DebconfFrontend::cmd_input },
-    { "GET", &DebconfFrontend::cmd_get },
-    { "CAPB", &DebconfFrontend::cmd_capb },
-    { "PROGRESS", &DebconfFrontend::cmd_progress },
-    { "X_PING", &DebconfFrontend::cmd_x_ping },
-    { "VERSION", &DebconfFrontend::cmd_version },
-    { "X_LOADTEMPLATEFILE", &DebconfFrontend::cmd_x_loadtemplatefile },
-    { "INFO", &DebconfFrontend::cmd_info },
-    { "FGET", &DebconfFrontend::cmd_fget },
-    { "FSET", &DebconfFrontend::cmd_fset },
-    { "BEGINBLOCK", &DebconfFrontend::cmd_beginblock },
-    { "ENDBLOCK", &DebconfFrontend::cmd_endblock },
-    { "STOP", &DebconfFrontend::cmd_stop },
-    { 0, 0 } };
+const DebconfFrontend::Cmd DebconfFrontend::commands[] = {{"SET", &DebconfFrontend::cmd_set, 2},
+                                                          {"GO", &DebconfFrontend::cmd_go, 0},
+                                                          {"TITLE", &DebconfFrontend::cmd_title, 1},
+                                                          {"SETTITLE", &DebconfFrontend::cmd_title, 1},
+                                                          {"DATA", &DebconfFrontend::cmd_data, 3},
+                                                          {"SUBST", &DebconfFrontend::cmd_subst, 3},
+                                                          {"INPUT", &DebconfFrontend::cmd_input, 1},
+                                                          {"GET", &DebconfFrontend::cmd_get, 1},
+                                                          {"CAPB", &DebconfFrontend::cmd_capb, 1},
+                                                          {"PROGRESS", &DebconfFrontend::cmd_progress, 1},
+                                                          {"X_PING", &DebconfFrontend::cmd_x_ping, 0},
+                                                          {"VERSION", &DebconfFrontend::cmd_version, 0},
+                                                          {"X_LOADTEMPLATEFILE", &DebconfFrontend::cmd_x_loadtemplatefile, 1},
+                                                          {"INFO", &DebconfFrontend::cmd_info, 0},
+                                                          {"FGET", &DebconfFrontend::cmd_fget, 2},
+                                                          {"FSET", &DebconfFrontend::cmd_fset, 3},
+                                                          {"BEGINBLOCK", &DebconfFrontend::cmd_beginblock, 0},
+                                                          {"ENDBLOCK", &DebconfFrontend::cmd_endblock, 0},
+                                                          {"STOP", &DebconfFrontend::cmd_stop, 0},
+                                                          {"REGISTER", &DebconfFrontend::cmd_register, 2},
+                                                          {"UNREGISTER", &DebconfFrontend::cmd_unregister, 1},
+                                                          {"EXIST", &DebconfFrontend::cmd_exist, 1},
+                                                          {nullptr, nullptr, 0}};
 
 DebconfFrontend::DebconfFrontend(QObject *parent)
-  : QObject(parent)
+    : QObject(parent)
 {
 }
 
@@ -108,15 +111,16 @@ void DebconfFrontend::setValue(const QString &key, const QString &value)
     m_values[key] = value;
 }
 
-template<class T> int DebconfFrontend::enumFromString(const QString &str, const char *enumName)
+template<class T>
+int DebconfFrontend::enumFromString(const QString &str, const char *enumName)
 {
     QString realName(str);
     realName.replace(0, 1, str.at(0).toUpper());
     int pos;
-    while ((pos = realName.indexOf(QLatin1Char( '_' ))) != -1) {
+    while ((pos = realName.indexOf(QLatin1Char('_'))) != -1) {
         if (pos + 1 >= realName.size()) { // pos is from 0, size from 1, mustn't go off-by-one
             realName.chop(1);
-        } else{
+        } else {
             realName.replace(pos, 2, realName.at(pos + 1).toUpper());
         }
     }
@@ -125,21 +129,22 @@ template<class T> int DebconfFrontend::enumFromString(const QString &str, const
     QMetaEnum e = T::staticMetaObject.enumerator(id);
     int enumValue = e.keyToValue(realName.toLatin1().data());
 
-    if(enumValue == -1) {
+    if (enumValue == -1) {
         enumValue = e.keyToValue(QString(QLatin1String("Unknown") + QLatin1String(enumName)).toLatin1().data());
-        qCDebug(DEBCONF) << "enumFromString (" <<QLatin1String(enumName) << ") : converted" << realName << "to" << QString(QLatin1String("Unknown") + QLatin1String(enumName)) << ", enum value" << enumValue;
+        qCDebug(DEBCONF) << "enumFromString (" << QLatin1String(enumName) << ") : converted" << realName << "to"
+                         << QString(QLatin1String("Unknown") + QLatin1String(enumName)) << ", enum value" << enumValue;
     }
     return enumValue;
 }
 
 DebconfFrontend::PropertyKey DebconfFrontend::propertyKeyFromString(const QString &string)
 {
-    return static_cast<PropertyKey>(enumFromString<DebconfFrontend>(string, "PropertyKey" ));
+    return static_cast<PropertyKey>(enumFromString<DebconfFrontend>(string, "PropertyKey"));
 }
 
 DebconfFrontend::TypeKey DebconfFrontend::type(const QString &string) const
 {
-    return static_cast<TypeKey>(enumFromString<DebconfFrontend>(property(string, Type), "TypeKey" ));
+    return static_cast<TypeKey>(enumFromString<DebconfFrontend>(property(string, Type), "TypeKey"));
 }
 
 void DebconfFrontend::reset()
@@ -163,21 +168,20 @@ QString DebconfFrontend::substitute(const QString &key, const QString &rest) con
 {
     Substitutions sub = m_subst[key];
     QString result, var, escape;
-    QRegExp rx(QLatin1String("^(.*)(\\\\)?\\$\\{([^\\{\\}]+)\\}(.*)$"));
+    QRegularExpression rx(QLatin1String("^(.*)(\\\\)?\\$\\{([^\\{\\}]+)\\}(.*)$"), QRegularExpression::DotMatchesEverythingOption);
     QString last(rest);
-    int pos = 0;
-    while ((pos = rx.indexIn(rest, pos)) != -1) {
-        qCDebug(DEBCONF) << "var found! at" << pos;
-        result += rx.cap(1);
-        escape = rx.cap(2);
-        var = rx.cap(3);
-        last = rx.cap(4);
+    QRegularExpressionMatch rxMatch = rx.match(rest);
+    while (rxMatch.hasMatch()) {
+        result += rxMatch.captured(1);
+        escape = rxMatch.captured(2);
+        var = rxMatch.captured(3);
+        last = rxMatch.captured(4);
         if (!escape.isEmpty()) {
-            result += QLatin1Literal("${") + var + QLatin1Char('}');
+            result += QLatin1String("${") + var + QLatin1Char('}');
         } else {
             result += sub.value(var);
         }
-        pos += rx.matchedLength();
+        rxMatch = rx.match(last);
     }
     return result + last;
 }
@@ -191,33 +195,34 @@ QString DebconfFrontend::property(const QString &key, PropertyKey p) const
     return r;
 }
 
-void DebconfFrontend::cmd_capb(const QString &caps)
+void DebconfFrontend::cmd_capb(const QStringList &args)
 {
+    const QString caps = args[0];
     Q_EMIT backup(caps.split(QLatin1String(", ")).contains(QLatin1String("backup")));
     say(QLatin1String("0 backup"));
 }
 
-void DebconfFrontend::cmd_set(const QString &param)
+void DebconfFrontend::cmd_set(const QStringList &args)
 {
-    const QString item = param.section(QLatin1Char(' '), 0, 0);
-    const QString value = param.section(QLatin1Char(' '), 1);
+    const QString item = args[0];
+    const QString value = args[1];
     m_values[item] = value;
     qCDebug(DEBCONF) << "# SET: [" << item << "] " << value;
     say(QLatin1String("0 ok"));
 }
 
-void DebconfFrontend::cmd_get(const QString &param)
+void DebconfFrontend::cmd_get(const QStringList &args)
 {
-    say(QLatin1String("0 ") + m_values.value(param));
+    say(QLatin1String("0 ") + m_values.value(args[0]));
 }
 
-void DebconfFrontend::cmd_input(const QString &param)
+void DebconfFrontend::cmd_input(const QStringList &args)
 {
-    m_input.append(param.section(QLatin1Char(' '), 1));
-    say(QLatin1String("0 will ask" ));
+    m_input.append(args[0]);
+    say(QLatin1String("0 will ask"));
 }
 
-void DebconfFrontend::cmd_go(const QString &)
+void DebconfFrontend::cmd_go(const QStringList &)
 {
     qCDebug(DEBCONF) << "# GO";
     m_input.removeDuplicates();
@@ -225,12 +230,65 @@ void DebconfFrontend::cmd_go(const QString &)
     m_input.clear();
 }
 
-void DebconfFrontend::cmd_progress(const QString &param)
+void DebconfFrontend::cmd_progress(const QStringList &args)
 {
+    const QString param = args[0];
     qCDebug(DEBCONF) << "DEBCONF: PROGRESS " << param;
     Q_EMIT progress(param);
 }
 
+void DebconfFrontend::cmd_register(const QStringList &args)
+{
+    const QString templateName = args[0];
+    const QString question = args[1];
+
+    if (!m_data.contains(templateName)) {
+        say(QLatin1String("20 No such template, \"%1\"").arg(templateName));
+        return;
+    }
+    m_data[question] = m_data[templateName];
+    say(QLatin1String("0 ok"));
+}
+
+void DebconfFrontend::cmd_unregister(const QStringList &args)
+{
+    const QString question = args[0];
+    if (!m_data.contains(question)) {
+        say(QLatin1String("20 %1 doesn't exist").arg(question));
+        return;
+    }
+    if (m_input.contains(question)) {
+        say(QLatin1String("20 %1 is busy, cannot unregister right now").arg(question));
+        return;
+    }
+    m_data.remove(question);
+    say(QLatin1String("0 ok"));
+}
+
+void DebconfFrontend::cmd_metaget(const QStringList &args)
+{
+    const QString question = args[0];
+    const QString fieldName = args[1];
+    if (!m_data.contains(question)) {
+        say(QLatin1String("20 %1 doesn't exist").arg(question));
+        return;
+    }
+    const auto properties = m_data[question];
+    const auto key = propertyKeyFromString(fieldName);
+    if (!properties.contains(key)) {
+        say(QLatin1String("20 %1 does not exist").arg(fieldName));
+        return;
+    }
+    const QString data = properties[key];
+    say(QLatin1String("0 ") + data);
+}
+
+void DebconfFrontend::cmd_exist(const QStringList &args)
+{
+    const QString question = args[0];
+    say(QLatin1String("0 ") + (m_data.contains(question) ? QLatin1String("true") : QLatin1String("false")));
+}
+
 void DebconfFrontend::next()
 {
     m_input.clear();
@@ -248,8 +306,9 @@ void DebconfFrontend::cancel()
     reset();
 }
 
-void DebconfFrontend::cmd_title(const QString &param)
+void DebconfFrontend::cmd_title(const QStringList &args)
 {
+    const QString param = args[0];
     if (!property(param, Description).isEmpty()) {
         m_title = property(param, Description);
     } else {
@@ -259,76 +318,77 @@ void DebconfFrontend::cmd_title(const QString &param)
     say(QLatin1String("0 ok"));
 }
 
-void DebconfFrontend::cmd_data(const QString &param)
+void DebconfFrontend::cmd_data(const QStringList &args)
 {
     // We get strings like
     // aiccu/brokername description Tunnel broker:
     // item = "aiccu/brokername"
     // type = "description"
     // rest = "Tunnel broker:"
-    const QString item = param.section(QLatin1Char(' '), 0, 0);
-    const QString type = param.section(QLatin1Char(' '), 1, 1);
-    const QString value = param.section(QLatin1Char(' '), 2);
+    const QString item = args[0];
+    const QString type = args[1];
+    const QString value = args[2];
 
     m_data[item][propertyKeyFromString(type)] = value;
     qCDebug(DEBCONF) << "# NOTED: [" << item << "] [" << type << "] " << value;
-    say(QStringLiteral( "0 ok" ));
+    say(QStringLiteral("0 ok"));
 }
 
-void DebconfFrontend::cmd_subst(const QString &param)
+void DebconfFrontend::cmd_subst(const QStringList &args)
 {
     // We get strings like
     // aiccu/brokername brokers AARNet, Hexago / Freenet6, SixXS, Wanadoo France
     // item = "aiccu/brokername"
     // type = "brokers"
     // value = "AARNet, Hexago / Freenet6, SixXS, Wanadoo France"
-    const QString item = param.section(QLatin1Char(' '), 0, 0);
-    const QString type = param.section(QLatin1Char(' '), 1, 1);
-    const QString value = param.section(QLatin1Char(' '), 2);
+    const QString item = args[0];
+    const QString type = args[1];
+    const QString value = args[2];
 
     m_subst[item][type] = value;
     qCDebug(DEBCONF) << "# SUBST: [" << item << "] [" << type << "] " << value;
     say(QLatin1String("0 ok"));
 }
 
-void DebconfFrontend::cmd_x_ping(const QString &param)
+void DebconfFrontend::cmd_x_ping(const QStringList &args)
 {
-    Q_UNUSED(param);
+    Q_UNUSED(args);
     say(QLatin1String("0 pong"));
 }
 
-void DebconfFrontend::cmd_version(const QString &param)
+void DebconfFrontend::cmd_version(const QStringList &args)
 {
-    if ( !param.isEmpty() ) {
+    const QString param = args[0];
+    if (!param.isEmpty()) {
         const QString major_version_str = param.section(QLatin1Char('.'), 0, 0);
         bool ok = false;
-        int major_version = major_version_str.toInt( &ok );
-        if ( !ok || (major_version != 2) ) {
+        int major_version = major_version_str.toInt(&ok);
+        if (!ok || (major_version != 2)) {
             say(QLatin1String("30 wrong or too old protocol version"));
             return;
         }
     }
-    //This debconf frontend is suposed to use the version 2.1 of the protocol.
+    // This debconf frontend is suposed to use the version 2.1 of the protocol.
     say(QLatin1String("0 2.1"));
 }
 
-void DebconfFrontend::cmd_x_loadtemplatefile(const QString &param)
+void DebconfFrontend::cmd_x_loadtemplatefile(const QStringList &args)
 {
-    QFile template_file(param);
+    QFile template_file(args[0]);
     if (template_file.open(QFile::ReadOnly)) {
         QTextStream template_stream(&template_file);
         QString line = QLatin1String("");
         int linecount = 0;
-        QHash<QString,QString> field_short_value;
-        QHash<QString,QString> field_long_value;
+        QHash<QString, QString> field_short_value;
+        QHash<QString, QString> field_long_value;
         QString last_field_name;
-        while ( !line.isNull() ) {
+        while (!line.isNull()) {
             ++linecount;
             line = template_stream.readLine();
             qCDebug(DEBCONF) << linecount << line;
-            if ( line.isEmpty() ) {
+            if (line.isEmpty()) {
                 if (!last_field_name.isEmpty()) {
-                    //Submit last block values.
+                    // Submit last block values.
                     qCDebug(DEBCONF) << "submit" << last_field_name;
                     const QString item = field_short_value[QLatin1String("template")];
                     const QString type = field_short_value[QLatin1String("type")];
@@ -338,8 +398,8 @@ void DebconfFrontend::cmd_x_loadtemplatefile(const QString &param)
                     m_data[item][DebconfFrontend::Type] = type;
                     m_data[item][DebconfFrontend::Description] = short_description;
                     m_data[item][DebconfFrontend::ExtendedDescription] = long_description;
-                    
-                    //Clear data.
+
+                    // Clear data.
                     field_short_value.clear();
                     field_long_value.clear();
                     last_field_name.clear();
@@ -349,11 +409,11 @@ void DebconfFrontend::cmd_x_loadtemplatefile(const QString &param)
                     last_field_name = line.section(QLatin1String(": "), 0, 0).toLower();
                     field_short_value[last_field_name] = line.section(QLatin1String(": "), 1);
                 } else {
-                    if ( field_long_value[last_field_name].isEmpty() ){
+                    if (field_long_value[last_field_name].isEmpty()) {
                         field_long_value[last_field_name] = line.remove(0, 1);
                     } else {
                         field_long_value[last_field_name].append(QLatin1Char('\n'));
-                        if ( !(line.trimmed() == QLatin1String(".")) ) {
+                        if (!(line.trimmed() == QLatin1String("."))) {
                             field_long_value[last_field_name].append(line.remove(0, 1));
                         }
                     }
@@ -367,22 +427,30 @@ void DebconfFrontend::cmd_x_loadtemplatefile(const QString &param)
     say(QLatin1String("0 ok"));
 }
 
-void DebconfFrontend::cmd_info(const QString &param)
+void DebconfFrontend::cmd_info(const QStringList &args)
 {
-    Q_UNUSED(param)
-    //FIXME: this is a dummy command, we should actually do something
-    //with param.
+    const QString &key = args[0];
+    if (key.isEmpty()) {
+        // this is the Perl debconf behaviour
+        say(QLatin1String("0 ok"));
+        return;
+    }
+    if (!m_data.contains(key)) {
+        say(QLatin1String("20 question doesn't exist"));
+        return;
+    }
+    m_side_info = m_data[key][PropertyKey::Description];
     say(QLatin1String("0 ok"));
 }
 
-void DebconfFrontend::cmd_fget(const QString &param)
+void DebconfFrontend::cmd_fget(const QStringList &args)
 {
     // We get strings like
     // foo/bar seen false
     // question = "foo/bar"
     // flag = "seen"
-    const QString question = param.section(QLatin1Char(' '), 0, 0);
-    const QString flag = param.section(QLatin1Char(' '), 1, 1);
+    const QString question = args[0];
+    const QString flag = args[1];
 
     if (m_flags[question][flag]) {
         say(QLatin1String("0 true"));
@@ -391,18 +459,18 @@ void DebconfFrontend::cmd_fget(const QString &param)
     }
 }
 
-void DebconfFrontend::cmd_fset(const QString &param)
+void DebconfFrontend::cmd_fset(const QStringList &args)
 {
     // We get strings like
     // foo/bar seen false
     // question = "foo/bar"
     // flag = "seen"
     // value = "false"
-    const QString question = param.section(QLatin1Char(' '), 0, 0);
-    const QString flag = param.section(QLatin1Char(' '), 1, 1);
-    const QString value = param.section(QLatin1Char(' '), 2, 2);
+    const QString question = args[0];
+    const QString flag = args[1];
+    const QString value = args[2];
 
-    if ( value == QLatin1String("false") ) {
+    if (value == QLatin1String("false")) {
         m_flags[question][flag] = false;
     } else {
         m_flags[question][flag] = true;
@@ -410,26 +478,26 @@ void DebconfFrontend::cmd_fset(const QString &param)
     say(QLatin1String("0 ok"));
 }
 
-void DebconfFrontend::cmd_beginblock(const QString &param)
+void DebconfFrontend::cmd_beginblock(const QStringList &args)
 {
-    Q_UNUSED(param)
-    //FIXME: this is a dummy command, we should actually do something
-    //with param.
+    Q_UNUSED(args)
+    m_making_block = true;
     say(QLatin1String("0 ok"));
 }
 
-void DebconfFrontend::cmd_endblock(const QString &param)
+void DebconfFrontend::cmd_endblock(const QStringList &args)
 {
-    Q_UNUSED(param)
-    //FIXME: this is a dummy command, we should actually do something
-    //with param.
+    Q_UNUSED(args)
+    if (m_making_block)
+        m_input.append(QStringLiteral("div"));
+    m_making_block = false;
     say(QLatin1String("0 ok"));
 }
 
-void DebconfFrontend::cmd_stop(const QString &param)
+void DebconfFrontend::cmd_stop(const QStringList &args)
 {
-     Q_UNUSED(param)
-     //Do nothing.
+    Q_UNUSED(args)
+    // Do nothing.
 }
 
 bool DebconfFrontend::process()
@@ -446,18 +514,38 @@ bool DebconfFrontend::process()
 
     qCDebug(DEBCONF) << "DEBCONF <--- [" << command << "] " << value;
     const Cmd *c = commands;
-    while (c->cmd != 0) {
+    while (c->cmd != nullptr) {
         if (command == QLatin1String(c->cmd)) {
-            (this->*(c->run))(value);
+            QStringList args{};
+            if (c->num_args_min > 0) {
+                int args_scanned = 0;
+                for (int i = 0; i < c->num_args_min; i++) {
+                    const int end_index = (i < (c->num_args_min - 1)) ? i : -1;
+                    const QString arg = value.section(QLatin1Char(' '), i, end_index);
+                    if (!arg.isEmpty())
+                        args_scanned++;
+                    args.append(arg);
+                }
+
+                if (args_scanned < c->num_args_min) {
+                    say(QLatin1String("20 Incorrect number of arguments"));
+                    return false;
+                }
+            } else {
+                args.append(value);
+            }
+            (this->*(c->run))(args);
             return true;
         }
-        ++ c;
+        ++c;
     }
+    say(QLatin1String("20 Unsupported command \"%1\" received").arg(command.toLower()));
     return false;
 }
 
 DebconfFrontendSocket::DebconfFrontendSocket(const QString &socketName, QObject *parent)
-  : DebconfFrontend(parent), m_socket(0)
+    : DebconfFrontend(parent)
+    , m_socket(nullptr)
 {
     m_server = new QLocalServer(this);
     QFile::remove(socketName);
@@ -490,7 +578,7 @@ void DebconfFrontendSocket::reset()
 {
     if (m_socket) {
         m_socket->deleteLater();
-        m_socket = 0;
+        m_socket = nullptr;
     }
     DebconfFrontend::reset();
 }
@@ -504,7 +592,7 @@ void DebconfFrontendSocket::cancel()
 }
 
 DebconfFrontendFifo::DebconfFrontendFifo(int readfd, int writefd, QObject *parent)
-  : DebconfFrontend(parent)
+    : DebconfFrontend(parent)
 {
     m_readf = new QFile(this);
     // Use QFile::open(fh,mode) method for opening read file descriptor as
@@ -530,7 +618,7 @@ void DebconfFrontendFifo::reset()
         m_readnotifier->setEnabled(false);
         m_readf->close();
         m_writef->close();
-        m_readf = m_writef = 0;
+        m_readf = m_writef = nullptr;
 
         // Use C library calls because QFile::close() won't close them actually
         ::close(readfd);
